@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2011-2021 Alexis Bienvenüe <paamc@passoire.fr>
+# Copyright (C) 2011-2025 Alexis Bienvenüe <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -411,14 +411,14 @@ sub populate_from_xml {
 
 sub define_statements {
     my ($self)     = @_;
-    my $t_page     = $self->table("page");
-    my $t_zone     = $self->table("zone");
-    my $t_position = $self->table("position");
-    my $t_failed   = $self->table("failed");
-    my $t_box        = $self->table( "box",       "layout" );
-    my $t_namefield  = $self->table( "namefield", "layout" );
-    my $t_layoutpage = $self->table( "page",      "layout" );
-    my $t_lnf        = $self->table( "namefield", "layout" );
+    my $t_page       = $self->table("page");
+    my $t_zone       = $self->table("zone");
+    my $t_position   = $self->table("position");
+    my $t_failed     = $self->table("failed");
+    my $t_box        = $self->table( "box", "layout" );
+    my $t_layoutzone = $self->table( "zone", "layout" );
+    my $t_layoutpage = $self->table( "page", "layout" );
+
     $self->{statements} = {
         NEWPageAuto => {
                 sql => "INSERT INTO $t_page"
@@ -567,7 +567,7 @@ sub define_statements {
             sql =>
 "SELECT enter.student AS student,enter.page AS page ,$t_page.copy AS copy"
               . " FROM (SELECT student,page FROM $t_box WHERE role=1"
-              . "       UNION SELECT student,page FROM $t_namefield) AS enter,"
+              . "       UNION SELECT student,page FROM $t_layoutzone) AS enter,"
               . "      $t_page"
               . " ON enter.student=$t_page.student"
               . " EXCEPT SELECT student,page,copy FROM $t_page"
@@ -579,7 +579,7 @@ sub define_statements {
         },
         noCapturePages => {
                 sql => "SELECT student,page,0 AS copy FROM $t_box WHERE role=1"
-              . " UNION SELECT student,page,0 AS copy FROM $t_namefield"
+              . " UNION SELECT student,page,0 AS copy FROM $t_layoutzone"
               . " EXCEPT SELECT student,page,copy FROM $t_page"
               . " WHERE timestamp_auto>0 OR timestamp_manual>0"
         },
@@ -830,7 +830,7 @@ sub define_statements {
               . "             b.image  AS image FROM"
               . " ( SELECT c.student,c.page,c.copy FROM"
               . "     (SELECT * FROM $t_page WHERE timestamp_auto>0 OR timestamp_manual>0 )"
-              . "      AS c, $t_lnf AS l"
+              . "      AS c, (SELECT * from $t_layoutzone WHERE zone='__n') AS l"
               . "     ON c.student=l.student AND c.page=l.page ) AS a"
               . " LEFT OUTER JOIN"
               . " ( SELECT student,page,copy,image FROM $t_zone WHERE type=? ) AS b"
@@ -857,6 +857,12 @@ sub define_statements {
               . "   AND z.type=? AND p.type=?"
               . " ORDER BY z.zoneid,p.corner"
         },
+        movePage => { sql =>
+              "UPDATE $t_page SET copy=? WHERE student=? AND page=? AND copy=?"
+          },
+        moveZones => { sql =>
+              "UPDATE $t_zone SET copy=? WHERE student=? AND page=? AND copy=?"
+          },
     };
     $self->{statements}->{pageSummary} =
       { sql => $self->{statements}->{pagesSummary}->{sql}
@@ -899,6 +905,21 @@ sub set_page_auto {
     }
 }
 
+# move_page_copy() moves page data corresponding to ($student,$page,$copy)
+# to ($student,$page,$copy_dest), and returns 1 if this erased previous data.
+
+sub move_page_copy {
+    my ( $self, $student, $page, $copy, $copy_dest ) = @_;
+    my $previous = 0;
+    if ( $self->get_page( $student, $page, $copy_dest ) ) {
+        $previous = 1;
+        $self->delete_page_data( $student, $page, $copy_dest );
+    }
+    $self->statement('movePage')->execute($copy_dest, $student, $page, $copy);
+    $self->statement('moveZones')->execute($copy_dest, $student, $page, $copy);
+    return ($previous);
+}
+
 # tag_overwritten(...) tags a page data as overwritten
 
 sub tag_overwritten {
@@ -906,7 +927,7 @@ sub tag_overwritten {
     $self->statement('overwritePage')->execute( $student, $page, $copy );
 }
 
-# sep_page_manual($student,$page,$copy,$timestamp) sets the timestamp
+# set_page_manual($student,$page,$copy,$timestamp) sets the timestamp
 # for a manual data capture for a particular page. With no $timestamp
 # argument, present time is used.
 
