@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2022 Alexis Bienvenüe <paamc@passoire.fr>
+# Copyright (C) 2008-2025 Alexis Bienvenüe <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -92,6 +92,8 @@ sub has_warnings_or_errors {
 sub format_markup {
     my ( $self, $t ) = @_;
     $t =~ s/\&/\&amp;/g;
+    $t =~ s/</\&lt;/g;
+    $t =~ s/>/\&gt;/g;
     return ($t);
 }
 
