@@ -29,7 +29,7 @@ GetOptions( "comment-only!" => \$comment_only );
 
 my ( $from, $to ) = @ARGV;
 
-$v = "1.5.2+git20221117172309 (r:716a08c3)";
+$v = "1.6.0+git20250411111559 (r:13e52b3d)";
 
 chomp( $date = `date "+%Y-%m-%d %H:%M %z"` );
 
@@ -51,7 +51,7 @@ if ( !$comment_only ) {
           . "Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
           . "Language-Team: LANGUAGE <LL@li.org>\n"
           . "MIME-Version: 1.0\n"
-          . "Content-Type: text/plain; charset=CHARSET\n"
+          . "Content-Type: text/plain; charset=UTF-8\n"
           . "Content-Transfer-Encoding: 8bit" );
 }
 
