@@ -1,6 +1,6 @@
 #! /usr/bin/env perl
 #
-# Copyright (C) 2012-2022 Alexis Bienvenüe <paamc@passoire.fr>
+# Copyright (C) 2012-2025 Alexis Bienvenüe <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -30,6 +30,7 @@ use AMC::DataModule::scoring qw(:question);
 use AMC::Scoring;
 use AMC::NamesFile;
 use AMC::Queue;
+use AMC::Topics;
 
 use Text::CSV;
 use File::Spec::Functions qw(tmpdir);
@@ -81,6 +82,7 @@ my $defaults = {
     annote_files        => [],
     annote_ascii        => 0,
     annote_position     => 'marge',
+    annote_color        => 'red',
     annotate_single     => '',
     verdict             => '%(id) %(ID)' . "\n" . 'TOTAL : %S/%M => %s/%m',
     verdict_question    => "\"%" . "s/%" . "m\"",
@@ -89,6 +91,7 @@ my $defaults = {
     ok_checksums_file   => '',
     to_check            => [],
     export_full_csv     => [],
+    export_full_ods     => [],
     export_columns      => 'student.copy',
     export_csv_ticked   => 'AB',
     export_csv_decimal  => ',',
@@ -100,6 +103,7 @@ my $defaults = {
     skip_scans          => 0,
     tracedest           => \*STDERR,
     debug_file          => '',
+    log_to              => '',
     pages               => '',
     extract_with        => 'qpdf',
     force_convert       => 0,
@@ -117,19 +121,19 @@ my $defaults = {
     exitonerror         => 1,
     postinstall         => '',
     additional_test     => '',
+    check_topics        => [],
+    move_files          => [],
 };
 
 sub new {
     my ( $class, %oo ) = @_;
 
-    my $self = { error=>0, %$defaults };
+    my $self = { error => 0, %$defaults };
 
     for ( keys %oo ) {
         $self->{$_} = $oo{$_} if ( exists( $self->{$_} ) );
     }
 
-    $self->{dir} =~ s:/[^/]*$::;
-
     bless( $self, $class );
 
     my $to_stdout = 0;
@@ -137,14 +141,12 @@ sub new {
     GetOptions(
         "debug!"         => \$self->{debug},
         "blind!"         => \$self->{blind},
-        "log-to=s"       => \$self->{debug_file},
+        "log-to=s"       => \$self->{log_to},
         "to-stdout!"     => \$to_stdout,
         "extract-with=s" => \$self->{extract_with},
         "tmp=s"          => \$self->{tmpdir},
     );
 
-    $self->{tmpdir} = tmpdir() if ( !$self->{tmpdir} );
-
     $self->{tracedest} = \*STDOUT if ($to_stdout);
     binmode $self->{tracedest}, ":utf8";
 
@@ -156,12 +158,16 @@ sub new {
 sub clean {
     my ($self) = @_;
 
-    $self->set( src => '', scans => '', names => '', list => '' );
+    $self->set( src => '', scans => '', names => '', list => '', error => 0 );
 }
 
 sub setup {
     my ($self)=@_;
 
+    $self->{dir} =~ s:/[^/]*$:: if(-f $self->{dir});
+
+    $self->{tmpdir} = tmpdir() if ( !$self->{tmpdir} );
+
     if ( !$self->{src} ) {
         opendir( my $dh, $self->{dir} )
           || die "can't opendir $self->{dir}: $!";
@@ -243,6 +249,15 @@ sub install {
     }
     closedir $sh;
 
+    for my $mv ( @{ $self->{move_files} } ) {
+        debug("Move $mv->{from} to $mv->{to}");
+        system(
+            "mv",
+            $self->{temp_dir} . '/' . $mv->{from},
+            $self->{temp_dir} . '/' . $mv->{to}
+        );
+    }
+
     print { $self->{tracedest} } "[>] Installed in $self->{temp_dir}\n";
 
     if ( -d ( $self->{temp_dir} . "/scans" ) && !$self->{scans} ) {
@@ -261,14 +276,17 @@ sub install {
     $self->{scans} = [] if ( !$self->{scans} );
 
     for my $d (
-        qw(data cr cr/corrections cr/corrections/jpg cr/corrections/pdf scans))
+        qw(data cr cr/corrections cr/corrections/jpg cr/corrections/pdf scans _build))
     {
         mkdir( $self->{temp_dir} . "/$d" ) if ( !-d $self->{temp_dir} . "/$d" );
     }
 
-    $self->{debug_file} = $self->{temp_dir} . "/debug.log"
-      if ( !$self->{debug_file} );
-    open( DB, ">", $self->{debug_file} );
+    if ( $self->{log_to} ) {
+        $self->{debug_file} = $self->{log_to};
+    } else {
+        $self->{debug_file} = $self->{temp_dir} . "/debug.log";
+    }
+    open( DB, ">>", $self->{debug_file} );
     print DB "Test\n";
     close(DB);
 
@@ -382,6 +400,7 @@ sub command {
 sub amc_command {
     my ( $self, $sub, @opts ) = @_;
 
+    push @opts, '--project-dir', '%PROJ';
     push @opts, '--debug', '%PROJ/debug.log' if ( $self->{debug} );
     @opts = map {
         s:%DATA:$self->{temp_dir}/data:g;
@@ -403,10 +422,11 @@ sub prepare {
         's[' . $self->{documents} . ']', '--epoch',
         946684800,                       '--n-copies',
         $self->{n_copies},               '--prefix',
-        $self->{temp_dir} . '/',         '%PROJ/' . $self->{src},
-        '--data',                        '%DATA',
+        $self->{temp_dir} . '/',         '--source',
+        '%PROJ/' . $self->{src},         '--data',
+        '%DATA',
     );
-    $self->amc_command( 'meptex', '--src', '%PROJ/calage.xy',
+    $self->amc_command( 'meptex',
         '--data', '%DATA', );
     $self->stop_clock("subject and layout");
 
@@ -417,7 +437,8 @@ sub prepare {
         $self->{tex_engine}, '--mode',
         'b',                 '--n-copies',
         $self->{n_copies},   '--data',
-        '%DATA',             '%PROJ/' . $self->{src},
+        '%DATA',             '--source',
+        '%PROJ/' . $self->{src},
     );
     $self->stop_clock("scoring strategy");
 }
@@ -427,7 +448,7 @@ sub analyse {
 
     $self->start_clock();
 
-    if ( ($self->{perfect_copy} || $self->{full_scans}) && !$oo{directory} ) {
+    if ( ( $self->{perfect_copy} || $self->{full_scans} ) && !$oo{directory} ) {
         $self->amc_command(
             'prepare',           '--filter',
             $self->{filter},     '--with',
@@ -435,7 +456,9 @@ sub analyse {
             'k',                 '--epoch',
             946684800,           '--n-copies',
             $self->{n_copies},   '--prefix',
-            '%PROJ/',            '%PROJ/' . $self->{src},
+            '%PROJ/',            '--data',
+            '%DATA',             '--source',
+            '%PROJ/' . $self->{src},
         );
     }
 
@@ -446,7 +469,7 @@ sub analyse {
         close(NUMS);
         $self->amc_command(
             'imprime',
-            '--sujet'         => '%PROJ/corrige.pdf',
+            '--sujet'         => '%PROJ/DOC-indiv-solution.pdf',
             '--methode'       => 'file',
             '--output'        => '%PROJ/xx-copie-%e.pdf',
             '--fich-numeros'  => $nf,
@@ -476,7 +499,7 @@ sub analyse {
             "-dBATCH"
         );
         push @cmd, "-dQUIET" if ( !$self->{debug} );
-        system( @cmd, $self->{temp_dir} . "/corrige.pdf" );
+        system( @cmd, $self->{temp_dir} . "/DOC-indiv-solution.pdf" );
 
         opendir( my $dh, $self->{temp_dir} )
           || die "can't opendir $self->{temp_dir}: $!";
@@ -579,7 +602,7 @@ sub analyse {
         '--tol-marque',
         $self->{tol_marque},
         ( $self->{ignore_red} ? '--ignore-red' : '--no-ignore-red' ),
-        '--projet',
+        '--project-dir',
         '%PROJ',
         '--data',
         '%DATA',
@@ -602,7 +625,7 @@ sub analyse {
                  $self->{debug}
               || $self->{debug_pixels} ? '--debug-pixels' : '--no-debug-pixels'
         ),
-        '--projet',
+        '--project-dir',
         '%PROJ', '--data', '%DATA',
         '--liste-fichiers',
         $scans_list,
@@ -738,8 +761,7 @@ sub check_perfect {
 
     for my $m ( @{ $self->{marks} } ) {
         $p{ $m->{student} } = 0
-          if ( $m->{total} == $m->{max}
-            && $m->{total} > 0 );
+          if ( $m->{total} == $m->{max} );
     }
 
     for my $i ( keys %p ) {
@@ -873,13 +895,13 @@ sub annote {
         '--verdict',          $self->{verdict},
         '--verdict-question', $self->{verdict_question},
         '--position',         $self->{annote_position},
+        '--text-color',       $self->{annote_color},
         '--project',          '%PROJ',
         '--data',             '%DATA',
         ( $self->{annote_ascii}
             ? "--force-ascii"
             : "--no-force-ascii" ),
         '--n-copies',              $self->{n_copies},
-        '--subject',               '%PROJ/sujet.pdf',
         '--src',                   '%PROJ/' . $self->{src},
         '--with',                  $self->{tex_engine},
         '--filename-model',        $self->{model},
@@ -1000,27 +1022,144 @@ sub defects {
     }
 }
 
+sub check_topics {
+    my ($self) = @_;
+    my @tests = @{ $self->{check_topics} };
+    if (@tests) {
+        my @checks = ();
+        my @errs   = ();
+
+        set_debug( $self->{debug_file} );
+
+        $self->begin( "Topics test (" . @tests . " values)" );
+        my $data   = AMC::Data->new( $self->{temp_dir} . "/data" );
+        my $topics = AMC::Topics->new(
+            project_dir => $self->{temp_dir},
+            data        => $data
+        );
+        $data->begin_read_transaction("ttop");
+        for my $test (@tests) {
+            if ( $test->{-copy} && !$test->{-student} ) {
+                if ( $test->{-copy} =~ /^(.*):(.*)$/ ) {
+                    $test->{-student} = $1;
+                    $test->{-copy}    = $2;
+                } else {
+                    $test->{-student} = $test->{-copy};
+                    $test->{-copy}    = 0;
+                }
+            }
+            my $id = $test->{-id};
+            my ($t) = grep { $_->{id} eq $id } ( $topics->all_topics );
+            if ($t) {
+                my $m = $topics->student_topic_message( $test->{-student},
+                                                        ($test->{-copy} || 0), $t );
+                my $i = 0;
+                for my $key (qw/message color/) {
+                    if ( defined( $test->{ "-" . $key } ) ) {
+                        push @checks,
+                          [
+                            $m->{$key},
+                            $test->{ "-" . $key },
+"Topic '$id' $key for copy ($test->{-student},$test->{-copy})"
+                          ];
+                        $i++;
+                    }
+                }
+                if ( !$i ) {
+                    push @errs, "[E] Empty test for topic '$id' and copy ($test->{-student},$test->{-copy})";
+                }
+            } else {
+                push @errs, "[E] Topic not found: $id";
+            }
+        }
+        $data->end_transaction("ttop");
+        $data->disconnect("topics");
+
+        set_debug('');
+
+        for my $c (@checks) {
+            $self->test(@$c);
+        }
+        for my $e (@errs) {
+            $self->trace($e);
+            $self->failed(1);
+        }
+
+        $self->end;
+    }
+}
+
 sub check_export {
     my ($self) = @_;
-    my @csv = @{ $self->{export_full_csv} };
+    $self->check_export_csv( key => 'export_full_csv' );
+    $self->check_export_csv( key => 'export_full_ods', via_ods => 1 );
+}
+
+sub check_export_csv {
+    my ($self, %opts) = @_;
+    my @csv = @{ $self->{$opts{key}} };
     if (@csv) {
-        $self->begin( "CSV full export test (" . ( 1 + $#csv ) . " scores)" );
+        $self->begin( ( $opts{via_ods} ? "ODS" : "CSV" )
+            . " full export test ("
+              . ( 1 + $#csv )
+              . " scores)" );
         my @args = (
-            '--data',            '%DATA',
-            '--module',          'CSV',
-            '--association-key', $self->{list_key},
-            '--option-out',      'columns=' . $self->{export_columns},
-            '--option-out',      'ticked=' . $self->{export_csv_ticked},
-            '--option-out',      'decimal=' . $self->{export_csv_decimal},
-            '-o',                '%PROJ/export.csv',
+            '--data',       '%DATA', '--association-key', $self->{list_key},
+            '--option-out', 'columns=' . $self->{export_columns},
         );
+        if ( $opts{via_ods} ) {
+            push @args, "--module", "ods", "-o", '%PROJ/export.ods';
+        } else {
+            push @args,
+              '--option-out', 'ticked=' . $self->{export_csv_ticked},
+              '--option-out', 'decimal=' . $self->{export_csv_decimal},
+              '--module',     'CSV',
+              '-o',           '%PROJ/export.csv';
+        }
         push @args, '--fich-noms', '%PROJ/' . $self->{list}
           if ( $self->{list} );
         $self->amc_command( 'export', @args );
+
+        my $skip_lines = 0;
+
+        if ( $opts{via_ods} ) {
+            my $ok;
+            {
+                local $ENV{LANG} = 'C';
+                $ok =
+                  run( ["unoconv", "-f", "csv", "$self->{temp_dir}/export.ods"],
+                    '2>>', '/dev/null' );
+            }
+            if ( !$ok ) {
+                $self->trace("[E] Command `unoconv' returned with $?");
+                $self->failed(1);
+            }
+            $skip_lines = 2;
+        }
+
         my $c = Text::CSV->new();
         open my $fh, "<:encoding(utf-8)", $self->{temp_dir} . '/export.csv';
-        my $i     = 0;
-        my %heads = map { $_ => $i++ } ( @{ $c->getline($fh) } );
+
+        if($skip_lines) {
+            for my $ii (1..$skip_lines) {
+                $c->getline($fh);
+            }
+        }
+        my %heads = ();
+        my $i=0;
+        my $h = $c->getline($fh);
+        while ( my ( $i, $h ) = each @$h ) {
+            if ( defined( $heads{$h} ) ) {
+                my $ni = 2;
+                while ( defined( $heads{ $h . "#$ni" } ) ) {
+                    $ni++;
+                }
+                $heads{ $h . "#$ni" } = $i;
+            } else {
+                $heads{$h} = $i;
+            }
+        }
+
         my $copy  = $heads{ translate_column_title('copie') };
         my $name  = $heads{ translate_column_title('nom') };
 
@@ -1036,10 +1175,16 @@ sub check_export {
             $irow++;
             for my $t (@csv) {
                 my $goodrow = '';
-                if ( $t->{-copy} && $t->{-copy} eq $row->[$copy] ) {
+                if (   $t->{-copy}
+                    && defined($copy)
+                    && $t->{-copy} eq $row->[$copy] )
+                {
                     $goodrow = 'copy ' . $t->{-copy};
                 }
-                if ( $t->{-name} && $t->{-name} eq $row->[$name] ) {
+                if (   $t->{-name}
+                    && defined($name)
+                    && $t->{-name} eq $row->[$name] )
+                {
                     $goodrow = 'name ' . $t->{-name};
                 }
                 if ( $t->{-irow} && $t->{-irow} == $irow ) {
@@ -1060,9 +1205,12 @@ sub check_export {
                     && defined( $heads{ $t->{-question} } )
                     && defined( $t->{-score} ) )
                 {
-                    $self->test( $row->[ $heads{ $t->{-question} } ],
+                    $self->test(
+                        $row->[ $heads{ $t->{-question} } ],
                         $t->{-score},
-                        "score for $goodrow Q=" . $t->{-question} );
+                        "score for $goodrow Q=" . $t->{-question},
+                        digits => $t->{-digits}
+                    );
                     $t->{checked} = 1;
                 }
                 if (   $goodrow
@@ -1242,7 +1390,7 @@ sub check_textest {
 
 sub check_subject {
     my ($self) = @_;
-    $self->see_file( $self->{temp_dir} . "/sujet.pdf" );
+    $self->see_file( $self->{temp_dir} . "/DOC-sujet.pdf" );
 }
 
 sub data {
@@ -1281,16 +1429,21 @@ sub datadump {
 }
 
 sub test {
-    my ( $self, $x, $v, $subtest ) = @_;
+    my ( $self, $x, $v, $subtest, %opts ) = @_;
     if ( !defined($subtest) ) {
         $subtest = ++$self->{'n.subt'};
     }
     if ( ref($x) eq 'ARRAY' ) {
         for my $i ( 0 .. $#$x ) {
-            $self->test( $x->[$i], $v->[$i], 1 );
+            $self->test( $x->[$i], $v->[$i], 1, %opts );
         }
     } else {
         no warnings 'uninitialized';
+        if ( defined( $opts{digits} ) && $opts{digits} ne '' ) {
+            for my $z ( \$x, \$v ) {
+                $$z = sprintf( "%.$opts{digits}f", $$z );
+            }
+        }
         if ( $x ne $v ) {
             $self->trace( "[E] "
                   . $self->{test_title}
@@ -1424,7 +1577,10 @@ sub report_uninitialized {
     my $u = 0;
     open( LOG, $self->{debug_file} );
     while (<LOG>) {
-        $u += 1 if ( /uninitialized/i && !/at \(eval [0-9]+\) line/ );
+        if ( /uninitialized/i && !/at \(eval [0-9]+\) line/ ) {
+            $u += 1 ;
+            print STDERR;
+        }
     }
     close LOG;
     $self->trace("[i] uninitialized: $u") if ($u);
@@ -1455,6 +1611,7 @@ sub default_process {
     $self->check_perfect;
     $self->check_assoc;
     $self->annote;
+    $self->check_topics;
     $self->check_export;
     $self->report_hardware;
 
