@@ -1,6 +1,6 @@
-#! /usr/bin/perl
+#! /usr/bin/env perl
 #
-# Copyright (C) 2013-2017 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2013-2025 Alexis Bienvenüe <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -20,10 +20,12 @@
 
 require "./AMC/Test.pm";
 
-AMC::Test->new('dir'=>__FILE__,
-	       list=>'students.csv',n_copies=>1,
-	       list_key=>'id',code=>'<preassoc>',
-	       check_assoc=>{3=>903},
-	      )
-  ->default_process;
+AMC::Test->new(
+    dir         => __FILE__,
+    list        => 'students.csv',
+    n_copies    => 1,
+    list_key    => 'id',
+    code        => '<preassoc>',
+    check_assoc => { 3 => 903 },
+)->default_process;
 
